﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Metrics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PIbd_22_Kalyshev_Y_V_MotorBoat_Base
{
    internal class SetBoatsGeneric<T>
    where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetBoatsGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="boat">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T boat)
        {
            for (int i = 0; i < _places.Length; i++)
            {
                if (_places[i] == null)
                {
                    _places[i] = boat;
                    return i;
                }
            }
            return -1;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="boat">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public bool Insert(T boat, int position)
        {
            if (position < 0 || position > _places.Length)
            {
                return false;
            }
            if (_places[position] != null)
            {
                int pos = -1;
                for (int i = position+1; i < _places.Length; i++)
                {
                    if (_places[i] == null)
                    {
                        pos = i;
                    }
                }
                if (pos != -1)
                {
                    for (int i = pos; i > position; i--)
                    {
                        _places[i] = _places[i - 1];
                    }
                } else
                {
                    return false;
                }
            }
            _places[position] = boat;
            return false;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position > _places.Length)
            {
                return null;
            }
            T removed = _places[position];
            _places[position] = null;
            return removed;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Get(int position)
        {
            if (position < 0 || position > _places.Length)
            {
                return null;
            }
            return _places[position];
        }
    }
}
